#pragma once
/**
\file BMPacket.h

Class and convenience functions for dealing with Bitmap datapackets.

*/
#include "common.h"
#include "GridPacket.h"
#include "globals.h"

#include "color.h"

class HField;

/** BMPacket is the specialized datatype used for passing RGB data around a world machine world.
 It is simply a specialization of WMGridPacket on the floating point RGB triple datatype. Look at WMGridPacket for more information

	@ingroup data
*/
class BMPacket :
	public WMGridPacket<rgb96_data_element>
{
public:
	static BMPointer CreateEmpty();
	static BMPointer CreateNew(const PacketContext &context);
	BMPacket &operator=(const BMPacket &other);

	static int getStaticType() { return WMP_rgb; };
	virtual int getType() { return getStaticType(); };
	virtual char *getTypeName() { return "Bitmap"; };
	virtual bool queryType(const int type);	

	virtual dword Color() { return 0x00ff0000; };

	virtual PacketPointer Clone() const;			// Return a copy of this WMPacket-derived packet
	virtual bool Load(std::istream &input);		// Load from stream input
	virtual bool Save(std::ostream &output);	// Save a copy of this data to stream output

	void ClampRange();				/// Clamp data to lie in the range 0..1.
/// Create a greyscale RGB packet from a heightfield
	bool CreateFromHeightField(HFPointer other);
	virtual void Clear();

protected:
	friend class WMPacketFactory;
	BMPacket();									// Protect constructor/destructor
	BMPacket(const BMPacket &other);			// copy construction
	virtual ~BMPacket();
private:
	typedef WMGridPacket<rgb96_data_element> super; 

};

/// Safe Accessor inline function to provide a safe way to cast from WMPacket
/// @relates BMPacket
inline BMPointer BMP(PacketPointer const &pack) { 
	if (pack) {
		bool ok = pack->getType() == WMP_rgb;
		if (!ok)
			return NULL;
	}
	return BMPointer(static_cast<BMPacket*>(pack.get()) ); 
};

/// Create a new bitmap; provide with a packet context containing desired size, etc.
/// It is recommended to simply use BMPacket::CreateNew() instead of this macro, which exists
/// only for backwards compatibility
/// @relates BMPacket
#define GetNewBMP(a)  (BMPacket::CreateNew(a))	

/// Create a new without initializing. You must call Init() on the returned packet before attempting
/// to call any function that accesses image data.
/// It is recommended to simply use BMPacket::CreateEmpty() instead of this macro
/// @relates BMPacket
#define GetEmptyBMP() (BMPacket::CreateEmpty())
