#if !defined(AFX_HFIELD_H__5095FCF1_CB94_4229_B855_0F89F8A35EE0__INCLUDED_)
#define AFX_HFIELD_H__5095FCF1_CB94_4229_B855_0F89F8A35EE0__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "common.h"
#include "GridPacket.h"
#include "globals.h"
#include <vector>


/** HField : The basic Heightfield class

	The backbone of most of World Machine, the heightfield class provides a basic regular heightfield grid.

	@ingroup data
 */
class HField : public WMGridPacket<float>
{
public:
	static HFPointer CreateEmpty();
	static HFPointer CreateNew(const PacketContext &context);
	virtual PacketPointer Clone() const;			// Return a copy of this WMPacket-derived packet
	HField &operator=(const HField &other);

	static int getStaticType() { return WMP_hfield; };
	virtual int getType() { return getStaticType(); };
	virtual char *getTypeName() { return "Heightfield"; };
	virtual bool queryType(const int type);	
	virtual dword Color() { return 0x00000; };

	virtual bool Load(std::istream &input);	// Load from stream input
	virtual bool Save(std::ostream &output);	// Save a copy of this data to stream output

// @name Additional basic HF Operations
//@{
	virtual void Clear();
	CoordF getRange() const;			///< Returns range of heightfield data, x=min, y=max
	void Normalize();					///< Rescale data to lie between 0..1
	void ClampRange();					///< Clamp data to lie 0..1. 
//@}

/// @name More advanced heightfield info functions
//@{
	float slope(const Coord loc);	///< Return slope as normalized angle (0..90) mapped to (0..1)
	float slopeF(const CoordF loc);	///< Same butbilinear sampled value
	// Return implicit normal vector 
	Vector2 slopeVector(const Coord &loc, bool zmultiply = true);
//@}

	friend class HFieldImporter;
	friend class HFieldExporter;

protected:
	friend class WMPacketFactory;
	HField();
	virtual ~HField();
private:
	typedef WMGridPacket<float> super; 
};

/// Safe Accessor inline function to provide a safe way to cast WMPacket to HField
/// in debug mode it will assert if the packet is not a HF
/// @relates HField
inline HFPointer HF(PacketPointer const & pack) { 
	if (pack) {
		bool ok = pack->getType() == WMP_hfield;
		if (!ok)
			return NULL;
	}
	return HFPointer((HField*) pack.get()); 
};

/// @relates HField
#define GetNewHF(a)	 (HField::CreateNew(a))
/// @relates HField
#define GetEmptyHF() (HField::CreateEmpty())


#endif // !defined(AFX_HFIELD_H__5095FCF1_CB94_4229_B855_0F89F8A35EE0__INCLUDED_)
