#ifndef _PLUGIN_MACROS_H
#define _PLUGIN_MACROS_H
/** @file PluginMacros.h
	@ingroup devices

	The macros that allow for easy device registration with the device host. Using this system, you don't have to manually hook up each device you write into the plumbing:
	The automatic system registers each device automatically with the device host in the executable or plugin you are creating.

	If you don't wish to use this, define NO_AUTOMATIC_DEVICES to disable the system.
*/

// Define this constant first thing in your project build settings or main include file to disable automatic registration
//#define NO_AUTOMATIC_DEVICES

#include "PluginHeader.h"
#include <functional>
#include <vector>

namespace WorldMachine {
namespace Plugins {

//////////////////////////////////////////////////////////////////////////

#ifndef NO_AUTOMATIC_DEVICES

typedef std::vector<DevPluginStruc> DEVICE_REGISTRY;
typedef std::vector<std::function<DevPluginStruc(void) > > DEVICE_DEFER_REGISTRY;

/*
#define REGISTER_DEVICE(nametag, datatype, deprecated )	 extern DEVICE_REGISTRY &getDeviceRegistryHost();\
														 DEVICE_LOCAL_INITIALIZER initializer##nametag(nametag, datatype, deprecated);
*/

template < class MakeFunc >
DevPluginStruc DEVICE_DEFERRED_INITIALIZER_FUNC(MakeFunc maker, int type, int datatype, int icon_index, bool deprecated) 
{ 
	Device *dev = maker();
	DevPluginStruc d;
	d.datatype = datatype; 
	d.type = type;
	d.deprecated = deprecated; strcpy(d.name, dev->GetTypeName()); d.lifedata = dev->GetLifeVars();
	d.icon_index = icon_index;
	dev->Destroy();
	return d;
};

template < Device* MakeFunc(), int TYPE, int DATA_TYPE, bool DEPREC, int ICON_INDEX = -1 >
class RunOnce {
public:
	RunOnce() {
		WorldMachine::Plugins::getDeviceRegistryHost().push_back( []() { return DEVICE_DEFERRED_INITIALIZER_FUNC(MakeFunc, TYPE, DATA_TYPE, ICON_INDEX, DEPREC); });
	};
};

#define WM_REGISTER_DEVICE_PLUGIN(makefunc, devtype, datatype, deprecated, iconindex ) namespace { WorldMachine::Plugins::RunOnce< makefunc, devtype, datatype, deprecated, iconindex > plugin_device_unique_registration_entry; };

extern DEVICE_DEFER_REGISTRY &getDeviceRegistryHost();

inline bool IterateDevices(DevPluginStruc *result, int i, int istart = 0) {
	DEVICE_DEFER_REGISTRY &dev = getDeviceRegistryHost();
	int index = i - istart;
	if (index < dev.size() && index >= 0) {
		*result = dev[index]();
		return true;
	}
	else
		return false;
}

}
} // end namespaces

#define CREATE_DEVICEHOST()  namespace WorldMachine { namespace Plugins { DEVICE_DEFER_REGISTRY &getDeviceRegistryHost() { static DEVICE_DEFER_REGISTRY registry; return registry; }; } }

#define DEFINE_WORLDMACHINE_PLUGIN() \
		extern "C" {\
			__declspec(dllexport) int  GetHeaderVersion();\
			__declspec(dllexport) void SetupGlobalAccess(WorldMachine::WMCoreInterface *global);\
			__declspec(dllexport) bool GetDevPlugin(const int i, DevPluginStruc *result);\
		};\


#endif
#else
// Create empty macros if not using automatic device creation
#define CREATE_DEVICEHOST()
#define DEFINE_WORLDMACHINE_PLUGIN()
#define REGISTER_DEVICE()

#endif