#pragma once

#include "wmpacket.h"
#include "hfield.h"
#include "bmpacket.h"

class HField;
class BMPacket;

/** TerrainCombo contains both a Heightfield and a RGB image.

In this version, they are required they be the same
resolution, although there is no reason they couldn't differ.

This is an internal datatype, as there really is no reason to pass them around combined like
this, at least for now.

*version 2.3: we're now adding ability for HF and BMP to be of different resolutions.

 in addition, in the future we might also add the ability to supply your own normal maps, and support
 mesh input in addition to HF input. We shall see!

	@ingroup data
*/
class TerrainComboPacket : public WMAreaPacket
{
public:
/// Get a new TerrainComboPacket
	static TCPointer CreateNew();
	virtual PacketPointer Clone() const;			// Return a copy of this WMPacket-derived packet
	TerrainComboPacket &operator=(const TerrainComboPacket &other);

	static int getStaticType() { return WMP_combo; };
	virtual int getType() override { return getStaticType(); };
	virtual char *getTypeName() { return "Terrain Combo"; };
	virtual bool queryType(const int type);	

	virtual dword Color() { return 0x00ff0000; };

	virtual size_t getHeapUsage() { size_t size = 0;  if (hf) size += hf->getHeapUsage(); if (bmp) size += bmp->getHeapUsage(); return size; };
	virtual bool isTypePageable() { return false; };

	virtual bool Load(std::istream &input);	
	virtual bool Save(std::ostream &output);	

/// @name Accessor Functions for sub packets
//@{
	bool valid() { return hf && bmp; };
	bool AssignHF(HFPointer const&_hf);
	bool AssignBMP(BMPointer const&_bmp);

	HFPointer const&getHF() { return hf; };
	BMPointer const&getBMP() { return bmp; };
//@}

	virtual void Clear();

protected:
	friend class WMPacketFactory;
	TerrainComboPacket(void);
	~TerrainComboPacket(void);

	virtual void resetContents();

	HFPointer hf;
	BMPointer bmp;
};

/** Packet Cast for TerrainComboPacket
 Accessor inline function to provide a safe way to cast WMPacket to TerrainComboPacket.
 In debug mode it will assert if the packet is not a TerrainComboPacket

 @relates TerrainComboPacket

 */
inline TCPointer TC(PacketPointer const & pack) { 
	if (pack) {
		bool ok = pack->getType() == WMP_combo;
		if (!ok)
			return NULL;
	}
	return (TerrainComboPacket*) pack.get(); 
};